#ifndef _MPI_SRIO_INTERFACES_H
#define _MPI_SRIO_INTERFACES_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>

#include "routingTableGen.h"
#define MPI_SRIO_MAX_PROCESS 180

typedef enum  {
  SRIO_CLUSTER_FORM_SUCCESS=SRIO_CLUSTER_FORMATION_SUCCESS,
  SRIO_CLUSTER_FORM_ERR_RTABLE_ENTRIES_FULL=SRIO_CLUSTER_ERR_RTABLE_ENTRIES_FULL,
  SRIO_CLUSTER_FORM_ERR_UNREACHABLE_NODE=SRIO_CLUSTER_ERR_UNREACHABLE_NODE
} SrioClusterFormRet_e;

/* Load the routing Data structures from the RoutingFile. The RoutingFile contains a collection of data structures 
    which contain SRIO routing table which can be loaded in to the memory before invoking other APIs. 
   Inputs: 
             config file_str : contains the file name where the routing tables are stored for this particular configuration.
   Returns:
              The base address of the memory to be used further by SRIO routing functions.
*/
void* SrioRoutingStructuresCreate(char *configFile_str/*IN*/);


/* Forms the Cluster with a set of cartridge physical location ids 
    The cluster is formed AFTER the routing structures are set up using SrioRoutingStructuresCreate(). Returns 0 if successful
Inputs:
         base: The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
         catridge_phys_ids[]: List of cartridge physical IDs[],  e.g. for c1,c4,c5 etc, the phys IDs would be 1,4,5 etc 
         num_catridge_phys_ids: Number of such physical IDs in the cluster
   Retuns:
        Returns SRIO_CLUSTER_FORM_SUCCESS(0) if successful
        Returns SRIO_CLUSTER_FORM_ERR_RTABLE_ENTRIES_FULL if routing tables were full on one of the nodes, and hence unsucccessful
        Returns SRIO_CLUSTER_FORM_ERR_UNREACHABLE_NODE  if some nodes in the cluster were not reachable, hence unsuccessful

*/
SrioClusterFormRet_e SrioFormCluster(void *base, int catridge_phys_ids[]/*IN*/, int num_catridge_phys_ids/*IN*/);

/* Get the SRIO ID from the physical location (eg C1N3 -> 3). Returns Status (SUCCESS/FAIL) along with the srioID.
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             cartridge_id: The cartridge id (i.e C2N3 --> cartridge ID = 2)
             node_id:       The cartridge id (i.e C2N3 --> Node ID = 3)

   Returns:
              The SRIO ID if successful, else returns -1
 */
int SrioGetSrioIDFromPhysID(void *base  /*IN*/, int cartridge_id  /*IN*/, int node_id /*IN*/,unsigned short *srioID /*OUT*/);


/* Gets the Physical ID from the SRIO ID (eg 3 -> C1N3). Returns Status (SUCCESS/FAIL) along with the physID 
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             srioID:             The SRIO ID whose PhysID (cartridge id & node id) needs to be found
             *cartridge_id : Pointer to the location where the cartridge id will be returned.
             *node_id : Pointer to the location where the node id will be returned.
   Returns:
              Returns 0 if successful, -1 if fails
*/ 
int SrioGetPhysIDFromSRIOID(void *base /*IN*/, unsigned short srioID, int *cartridge_id/*OUT*/,int *node_id/*OUT*/);

/* Return the physical port to route SRIO packets originating from srioID_src to srioID_dest 
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             srioID_src : The SRIO ID to route from.
             srioID_dest : The SRIO ID to route to.
   Returns:
              The routing port to route packets bound to srioID_dest, from srioID_src
*/
int SrioGetRoutingPort(void *base, unsigned short srioID_src  /*IN*/, unsigned short srioID_dest  /*IN*/);

/* Displays the cluster Routing Structures 
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
   Returns:
             0 if successful
*/

int SrioDisplayClusterInformation(void *base);

/* Resets the cluster Routing Structures. Typically disables all the cartidges in the topology. Used to 
    restart routing algorithm without reloading the topology.  
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
   Returns:
              0 if successful.
*/
int SrioResetCluster(void *base);


#ifdef DEVICE_K2H 
/* Resets the packet forwarding registers of the DEVICE_K2H  node
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             srio_base_virtual : The virtually mapped base address of SRIO registers, mapped via UIO
             srio_id : The SRIO Node id for which the packet forwarding registers need to be reset
   Returns:
              0 if successful.
*/
int SrioResetPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id);

/* Program the packet forwarding registers of the DEVICE_K2H  node, as per the routing tables stored in 'base'
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             srio_base_virtual : The virtually mapped base address of SRIO registers, mapped via UIO
             srio_id : The SRIO Node id for which the packet forwarding registers need to be programmed
   Returns:
              0 if successful.
*/
int SrioProgramPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id);

/* Display the packet forwarding registers of the DEVICE_K2H  node, as per the routing tables stored in 'base'
   Inputs: 
             base : The base address allocated by SrioRoutingStructuresCreate(), to be used during routing lookup.
             srio_base_virtual : The virtually mapped base address of SRIO registers, mapped via UIO
             srio_id : The SRIO Node id for which the packet forwarding registers need to be displayed
   Returns:
              0 if successful.
*/
int SrioDisplayPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id);
#endif

void *SrioHostnameListCreate (char *file_hostname);
/* Return the physical location (cartridge_id and node_id) using hostname
   Inputs:
             base : list of hostnames and its cartridge and node IDs

   Outputs:
             cartridge_id: cartridge_id
             node_id: node_id
   Returns:
             0 for success, -1 for error (hostname not found in list)
*/
int SrioHostname2PhysID (void *hostnameList, char *hostname, int *dest_cartridge_id, int *dest_node_id);
/* Return the neigbouring nodes SRIO IDs connected to all the 4 ports and populate them in output_dest_ids[port] array
 * If neighbour is not present or not enabled in the cluster, a -1 is populated in the output_dest_ids[port] */
int SrioFindMyNeighbours(void *base,int srioID,int output_dest_ids[],int only_active_nodes);

/**************************************************/
/* 7 APIs needed for integration into MPI         */
/**************************************************/
extern void *Srio_Type11_init (void *baseAddrOfRoutingTbl, int cartridge_id, int node_id);
extern int Srio_Type11_setup (void *drvHandle, void *baseAddrOfRoutingTbl, int list_of_srio_ids[], int num_of_srio_ids);
extern void *Srio_Type11_AcquireTX (void *drvHandle, void **ptr, int *buff_len, int srio_dest_id); /* returns cppi_desc */
extern int Srio_Type11_SendTX (void *drvHandle, void *cppi_desc, int srio_dest_id, int num_bytes); /* routing tag is already inserted into buffer, first location */
extern void *Srio_Type11_AcquireRX(void *drvHandle, void **ptr, int *buff_len); /* returns cppi_desc */
extern void Srio_Type11_ReleaseRX(void *drvHandle, void *cppi_desc);
int Srio_Type11_finalize (void *drvHandle);

typedef struct {
  int cartridge_id;
  int node_id;
  char hostname[80];
} SrioHostList_t;

#endif
/*** nothing past this point ***/









